/* eslint-disable no-underscore-dangle */
import React from 'react';
import PropTypes from 'prop-types';
import { Router4Compat as Router } from 'react-router-4-compat';
import { LocaleProvider, Spin } from 'antd';
import zhCN from 'antd/lib/locale-provider/zh_CN';
import { PersistGate } from 'redux-persist/integration/react';
import isString from 'lodash/isString';
import config from './utils/config';
import { getUser, isAuthed, histories } from './utils/auth';
import { processError } from './utils/error';
import App from './routes/app';
import { getMenus, getModuleInfo, getModuleLayout } from './data/modules';
import { bindModel } from './utils/model';
import Monk from './routes/main/monk';
import { createPersistor } from './data/app';

import styles from './index.css';

const Loading = <Spin size="large" className={styles.globalSpin} />;

const registerModel = (app, model) => {
  // noinspection JSUnresolvedVariable
  if (!(app._models.filter(m => m.namespace === model.namespace).length === 1)) {
    app.model(model);
  }
};

const maybeLogin = async (replace) => {
  if (!(await isAuthed())) {
    return replace('/login');
  }
};

const authenticated = async (replace) => {
  await maybeLogin(replace);
};

const combinePath = (base, path) => {
  if (!base) {
    return path;
  }
  if (base[base.length - 1] === '/') {
    return `${base}${path}`;
  } else {
    return `${base}/${path}`;
  }
};

const createRoute = async (app, group, basePath) => {
  const { name, showName, modules, children } = group;
  const theFullPath = combinePath(basePath, name);
  // noinspection JSUnusedLocalSymbols
  return {
    path: name,
    fullPath: theFullPath,
    name: showName,
    component: Monk,
    getChildRoutes(nextState, cb) {
      createRoutes(app, modules, children, theFullPath)
        .then((result) => {
          cb(null, result);
        })
        .catch((err) => {
          cb(err, null);
        });
    },
  };
};

const moduleLeaveHook = (app, module) => {
  const models = (app._models || []).filter(m => m.namespace.startsWith(`${module.name}/`));
  const store = app._store;
  models.forEach((m) => {
    const { reducers, effects } = m;
    if ((reducers && reducers['@@exit']) || (effects && effects['@@exit'])) {
      store.dispatch({ type: `${m.namespace}/@@exit`, payload: module });
    }
    store.dispatch({ type: `${m.namespace}/@@reset` });
  });
};

const moduleEnterHook = (app, uid, module) => {
  const models = (app._models || []).filter(m => m.namespace.startsWith(`${module.name}/`));
  const store = app._store;
  models.forEach((m) => {
    const { reducers, effects } = m;
    if ((reducers && reducers['@@enter']) || (effects && effects['@@enter'])) {
      store.dispatch({ type: `${m.namespace}/@@enter`, payload: module });
    }
  });
  histories.pushHistory('module', uid, module);
};

const createRoutes = async (app, modules, groups, basePath) => {
  const routes = [];
  if (modules) {
    for (const module of modules) {
      let info;
      let layout;
      if (isString(module)) {
        info = await getModuleInfo(module);
        layout = await getModuleLayout(module);
      } else {
        info = module;
        layout = module.layout;
      }
      const { name, showName } = info;
      const route = {
        path: name,
        fullPath: combinePath(basePath, name),
        name: showName,
      };
      if (layout.route || layout.template) {
        let routeBundle = await import(`./routes/main/modules/${layout.route || layout.template}`);
        routeBundle = routeBundle.default || routeBundle;
        const binder = bindModel(app, info, layout);
        routeBundle = routeBundle(binder, info, layout);
        route.component = routeBundle;
        if (routeBundle.route) {
          for (const key in routeBundle.route) {
            // noinspection JSUnfilteredForInLoop
            if ({}.hasOwnProperty.call(routeBundle.route, key)) {
              // noinspection JSUnfilteredForInLoop
              route[key] = routeBundle.route[key];
            }
          }
        }
      } else {
        route.component = Monk;
      }
      const infoEx = {
        ...info,
        path: route.fullPath,
      };
      const { onLeave } = route;
      if (onLeave) {
        route.onLeave = (preState) => {
          moduleLeaveHook(app, infoEx);
          onLeave(preState);
        };
      } else {
        route.onLeave = () => {
          moduleLeaveHook(app, infoEx);
        };
      }
      if (route.onEnter) {
        const onEnter = route.onEnter;
        route.onEnter = (nextState, replace, cb) => {
          getUser()
            .then(u => u.id)
            .then(uid => moduleEnterHook(app, uid, infoEx))
            .then(() => new Promise((resolve, reject) => {
              onEnter(nextState, replace, (err, res) => {
                if (err) {
                  reject(err);
                } else {
                  resolve(res);
                }
              });
            }))
            .then(() => cb())
            .catch(err => cb(err));
        };
      } else {
        route.onEnter = (nextState, replace, cb) => {
          getUser()
            .then(u => u.id)
            .then(uid => moduleEnterHook(app, uid, infoEx))
            .then(() => cb())
            .catch(err => cb(err));
        };
      }
      routes.push(route);
    }
  }
  if (groups) {
    for (const group of groups) {
      routes.push(await createRoute(app, group, basePath));
    }
  }
  return routes;
};

function RouterConfig({ history, app }) {
  const routes = [
    {
      path: '/',
      component: App,
      indexRoute: {
        onEnter: (nextState, replace, cb) => {
          getUser()
            .then(u => u && u.id)
            .then(uid => histories.getLatest('module', uid))
            .then((latest) => {
              if (latest && config.fastNavigationPage) {
                replace('/fastNav');
              } else {
                replace('/main');
              }
              cb();
            })
            .catch((err) => {
              cb(err);
            });
        },
      },
      childRoutes: [
        {
          path: 'login',
          getComponent(ignored, cb) {
            Promise.all([
              import('./models/login'),
              import('./routes/login'),
            ]).then(([model, route]) => {
              registerModel(app, model.default);
              cb(null, route.default);
            });
          },
        },
        {
          path: 'main',
          onEnter: (ignored, replace, cb) => {
            authenticated(replace).then(() => cb()).catch(err => cb(err));
          },
          getComponent(ignored, cb) {
            Promise.all([
              import('./models/main'),
              import('./routes/main'),
            ]).then(([model, route]) => {
              registerModel(app, model.default);
              cb(null, route.default);
            });
          },
          getChildRoutes: (nextState, cb) => {
            getMenus()
              .then((menus) => {
                createRoutes(app, [], menus, '/main')
                  .then((result) => {
                    cb(null, result);
                  })
                  .catch((err) => {
                    cb(err, null);
                  });
              })
              .catch((err) => {
                cb(err, null);
              });
          },
        },
      ],
    },
  ];
  if (config.fastNavigationPage) {
    routes[0].childRoutes.push({
      path: 'fastNav',
      onEnter: (ignored, replace, cb) => {
        authenticated(replace).then(() => cb()).catch(err => cb(err));
      },
      getComponent(ignored, cb) {
        Promise.all([
          import(`./models/main/modules/${config.fastNavigationPage}`),
          import(`./routes/main/modules/${config.fastNavigationPage}`),
        ]).then(([model, route]) => {
          registerModel(app, model.default);
          cb(null, route.default);
        });
      },
    });
  }
  return (
    <PersistGate persistor={createPersistor(app)} loading={Loading}>
      <LocaleProvider locale={zhCN}>
        <Router history={history} routes={routes} onError={processError} />
      </LocaleProvider>
    </PersistGate>
  );
}

RouterConfig.propTypes = {
  history: PropTypes.object.isRequired,
  app: PropTypes.object.isRequired,
};

export default RouterConfig;
