/** @module app */

import isNumber from 'lodash/isNumber';
import isFinite from 'lodash/isFinite';
import isString from 'lodash/isString';
import request from '../utils/request';
import post from '../utils/post';
import config from '../utils/config';

/**
 * @typedef {Object} RestResponse
 * @template T
 * @property {number} errorCode 错误码，0表示成功
 * @property {T} data 数据
 */

/**
 * @typedef {Object} DeploymentInfo 部署信息
 * @property {number} id 部署id
 * @property {string} versionNumber 版本号
 * @property {string} description 描述，一般为更新说明
 * @property {Date} updateTime 部署发布时间
 * @property {string} uri 资源内部uri，表示app安装包文件的内部定位地址
 * @property {string} status 部署状态，可能值为release，development，broken。release表示公开版本，development表示内部测试版，broken表示有重大bug，短时间不能解决，需要回滚
 */

/**
 * @typedef {Object} AppInfo app信息
 * @property {string} name 名称
 * @property {string} description 描述
 * @property {string} packageName 包名
 * @property {Date} createTime 创建时间
 * @property {Array.<DeploymentInfo>} history 历史部署信息
 */

/**
 * 根据名称或许符合条件的app相关信息，其中也包括部署信息
 * @param {!string} name 对app名字的筛选条件，支持模糊查询，为空时等价于查全部
 * @param {?number} pst 分页的开始位置，-1代表不分页
 * @param {?number} psz 分页的大小
 * @returns {Promise.<RestResponse.<Array.<AppInfo>>>} 所有符合条件的app信息
 */
export const getAppInfoes = async (name, pst = -1, psz = -1) => {
  return request(`${config.apiContextPath}/api/app/admin/apps/list`, { name, pst, psz });
};

/**
 * 新增app
 * @param {!string} name 名称
 * @param {?string} packageName
 * @param {?string} description
 * @returns {Promise.<RestResponse.<AppInfo>>} 新增的app信息
 */
export const addApp = async (name, packageName, description) => {
  return post(`${config.apiContextPath}/api/app/admin/apps/create`, { name, package: packageName, description });
};

/**
 * 更新app信息，忽略undefined的字段
 * @param {!number|string} idOrName app的id或名称，若为数字，则认为是id，字符串则认为是名称
 * @param {?string} newName 新的名称
 * @param {?string} packageName 包名
 * @param {?string} description 描述
 * @returns {Promise.<RestResponse.<null>>}
 */
export const editApp = async (idOrName, newName, packageName, description) => {
  if (isNumber(idOrName) && !isFinite(idOrName)) {
    return post(`${config.apiContextPath}/api/app/admin/apps/edit`, { id: idOrName, newName, package: packageName, description });
  } else if (isString(idOrName)) {
    return post(`${config.apiContextPath}/api/app/admin/apps/edit`, { name: idOrName, newName, package: packageName, description });
  } else {
    throw new Error(`Invalid id or name: ${idOrName}`);
  }
};

/**
 * 删除app，所有部署包会一同删除
 * @param {!number|string} idOrName app的id或名称，若为数字，则认为是id，字符串则认为是名称
 * @returns {Promise.<RestResponse.<null>>}
 */
export const removeApp = async (idOrName) => {
  if (isNumber(idOrName) && !isFinite(idOrName)) {
    return post(`${config.apiContextPath}/api/app/admin/apps/delete`, { id: idOrName });
  } else if (isString(idOrName)) {
    return post(`${config.apiContextPath}/api/app/admin/apps/delete`, { name: idOrName });
  } else {
    throw new Error(`Invalid id or name: ${idOrName}`);
  }
};

/**
 * 部署app，必须是一个新版本，若版本号低于或等于已部署的最高版本，则会部署失败
 * @param {!number|string} idOrName app的id或名称，若为数字，则认为是id，字符串则认为是名称
 * @param {!string} uri apk上传后得到的内部uri
 * @param {?string} description 更新说明
 * @param {?boolean} release 是否release版本，默认true
 * @returns {Promise.<RestResponse.<DeploymentInfo>>} 部署信息
 */
export const deployApp = async (idOrName, uri, description, release) => {
  if (isNumber(idOrName) && !isFinite(idOrName)) {
    return post(`${config.apiContextPath}/api/app/admin/apps/deploy`, { id: idOrName, uri, description, release });
  } else if (isString(idOrName)) {
    return post(`${config.apiContextPath}/api/app/admin/apps/deploy`, { name: idOrName, uri, description, release });
  } else {
    throw new Error(`Invalid id or name: ${idOrName}`);
  }
};

/**
 * 反部署app
 * @param {!number} id 部署app后返回的部署信息中记录的部署id
 * @returns {Promise.<RestResponse.<null>>}
 */
export const undeployApp = async (id) => {
  return post(`${config.apiContextPath}/api/app/admin/apps/undeploy`, { id });
};

/**
 * 修改部署信息
 * @param {!number} id 部署app后返回的部署信息中记录的部署id
 * @param {?string} description 部署描述，即更新说明
 * @param {?string} status 部署状态，可能值为release，development，broken。release表示公开版本，development表示内部测试版，broken表示有重大bug，短时间不能解决，需要回滚
 * @returns {Promise.<RestResponse.<null>>}
 */
export const editDeployment = async (id, description, status) => {
  return post(`${config.apiContextPath}/api/app/admin/apps/deployments/edit`, { id, description, status });
};
